clear();

z=300;
u  = 0.2*ones(z,1);
t  = linspace(0,3,z);


s = poly(0,"s");
pa=0.0246580;
pb=7.9769475;
pc=280.08032;
G = syslin('c',[1],[pa*s*s+pb*s+pc]);
u = 300*ones(z,1);

y   = csim(u',t',G);
//plot(t,y(1,:),t,om);


disp("Mit zusätzlichem Integrator um Übertragungsfunktion für Winkel zu bekommen:");

GI = syslin('c',[1],[pa*s*s*s+pb*s*s+pc*s]);
yi = csim(u',t',GI);

//plot(t,yi(1,:),'re',t,phi,'gre',m1(:,1),-m1(:,2)+0.12,'blu');

rp=1200.0;
ri=16.0;

//R=rp+ri/s=(rp*s+ri)/s

R = syslin('c',[rp*s+ri],[s]);

H = R*GI;
Q = H/(1+H);

z=300;
u  = 0.2*ones(z,1);
t  = linspace(0,3,z);
yq = csim(u',t',Q);

//plot(t,yq);

disp("Darstellung und Simulation von G im Zeitbereich");

Ag=[0,1;-pc/pa,-pb/pa];
Bg=[0;1/pa];

//Vergleichskontrolle der Eigenwerte:
zust=tf2ss(G);
Ag_test=zust(2);
disp(spec(Ag));
disp(spec(Ag_test));

disp("Simulationsmodell für G:");

function f = rechteSeiteG(t,yy)
    u_sim = 300.0;
    f = Ag*yy+Bg*u_sim; 
endfunction

y0 = [0,0]';
t0 = 0;
yg_sim  = ode(y0,t0,t,rechteSeiteG);

disp("O.K.: liegen ineinander");
//BEI BEDARF ANZEIGEN LASSEN:
//plot(t,yg_sim(1,:),'gre',t,y,'red--');

//------------------------------
//GI im Zeitbereich abbilden:

Agi=[   0, 1,    0;
        0, 0,    1;
        0,-pc/pa,-pb/pa];
Bgi=[0;0;1/pa];

disp("Simulationsmodell für GI:");

function f = rechteSeiteGI(t,yy)
    u_sim = 300.0;
    f = Agi*yy+Bgi*u_sim; 
endfunction

y0 = [0,0,0]';
t0 = 0;
ygi_sim  = ode(y0,t0,t,rechteSeiteGI);

disp("O.K.: liegen ineinander");
//BEI BEDARF ANZEIGEN LASSEN:
//plot(t,ygi_sim(1,:),'gre',t,yi,'red--');

//Auch hier die Eigenwerte vergleichen:
zusti=tf2ss(GI);
Agi_test=zusti(2);
disp(spec(Agi));
disp(spec(Agi_test));

//------------------------------
//GI mit PI-Regler im Zeitbereich abbilden:
//Jetzt Integral von phisoll - phi mitführen:
//(phisoll=0.2)
PHISOLL=0.2;
Agii=[   0,-1, 0,    0;
         0, 0, 1,    0;
         0, 0, 0,    1;
         0, 0,-pc/pa,-pb/pa];

Bgii=[1,0,0,0;
      0,0,0,0;
      0,0,0,0;
      0,0,0,1/pa];

rp=1200.0;
ri=16.0;

function f = rechteSeiteGIR(t,yy)
    w       = PHISOLL-yy(2);
    w_integ = yy(1); 

    u_begrenzt = rp*w + ri*w_integ;

    if u_begrenzt>500 then
        u_begrenzt=500;
    end

    if u_begrenzt<-500 then
        u_begrenzt=-500;
    end

    u_sim   = [PHISOLL;0;0;u_begrenzt]; 
    f = Agii*yy+Bgii*u_sim; 
endfunction

y0 = [0,0,0,0]'; 
t0 = 0;
ygir_sim  = ode(y0,t0,t,rechteSeiteGIR);

//plot(t,ygir_sim(2,:),'gre',t,yq,'red--');

//-----------------------------------
//Optimierung:

z=300;
u  = 0.2*ones(z,1);
t  = linspace(0,3,z);

function err=berechneFehler(x)
    rp=x(1);
    ri=x(2);

    y0 = [0,0,0,0]'; 
    t0 = 0;
    ygir_sim  = ode(y0,t0,t,rechteSeiteGIR);

    //Achtung: phi ist der zweite Zustand (der erste ist das Integral von -phi).
    err = sum((ygir_sim(2,:)-u').*(ygir_sim(2,:)-u'));
endfunction

rp=1200.0;
ri=16.0;

x0=[rp;ri];

[fopt, xopt, gopt] = optim(list(NDcost,berechneFehler), "b", x0.*0.001, x0.*1000.0, x0);

rp=xopt(1);
ri=xopt(2);
y0 = [0,0,0,0]'; 
t0 = 0;
ygir_simopt  = ode(y0,t0,t,rechteSeiteGIR);

plot(t,ygir_sim(2,:),'gre',t,ygir_simopt(2,:),'red');

disp(rp);
disp(ri);
